/**
* @file common_itk_util.h
* @brief itk warpper utility function 
* @author Ray
* @history
* ===================================================================================
* Date             Name                   Description of Change
* 09-July-2008     Ray                  
*/

#ifndef TC_LOG_H
#define TC_LOG_H

#ifdef __cplusplus
extern "C" {
#endif

void CreateTcLogFile(const char* logFileName);
void WriteTcLog(const char* format, ...);
void WriteTcLog2(string msg);
void CloseTcLog(void);
void log_open(void);
void log_close(void);
string getNow(char* format);
//int FindDatasetReferenceExt( tag_t datasettype, const char *datasettype_ref, char ext[10] );
//int CompareDate( date_t date1, date_t date2 );
//int GetRandomTempFile( char tempFile[256] );
//logical IsItemRevisionType( char object_type[WSO_name_size_c + 1] );
string getDateString(date_t date);

#ifdef __cplusplus
}
#endif

#endif